/*
  ZynAddSubFX - a software synthetizer
 
  FilterParams.C - Parameters for filter
  Copyright (C) 2002-2003 Nasca Octavian Paul
  Author: Nasca Octavian Paul

  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License 
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License (version 2) for more details.

  You should have received a copy of the GNU General Public License (version 2)
  along with this program; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/

#include <math.h>
#include <stdio.h>
#include "FilterParams.h"

FilterParams::FilterParams(unsigned char Ptype_,unsigned char Pfreq_,unsigned  char Pq_){
    Ptype=Ptype_;
    Pfreq=Pfreq_;
    Pq=Pq_;
    Pstages=0;
    Pfreqtrack=64;
};

FilterParams::~FilterParams(){
};


/*
 * Parameter control
 */
REALTYPE FilterParams::getfreq(){
    return((Pfreq/64.0-1.0)*5.0);
};

REALTYPE FilterParams::getq(){
    return(exp(pow((REALTYPE) Pq/127.0,2)*log(1000))-0.9);
};
REALTYPE FilterParams::getfreqtracking(REALTYPE notefreq){
    return(pow(notefreq/440.0,(Pfreqtrack-64.0)/64.0));
};


/*
 * Save or load the parameters to/from the buffer
 */
void FilterParams::saveloadbuf(Buffer *buf){
    unsigned char npar,n,tmp;

#ifdef DEBUG_BUFFER
    fprintf(stderr,"\n( Filter parameters) \n");
#endif    

    tmp=0xfe;
    buf->rwbyte(&tmp);//if tmp!=0xfe error


    for (n=0x80;n<0xf0;n++){
	if (buf->getmode()==0) {
	    buf->rwbyte(&npar);
	    n=0;//force a loop until the end of parameters (0xff)
	} else npar=n;
	if (npar==0xff) break;
	
	switch(npar){
	    //FILTER parameters
	    case 0x80:	buf->rwbytepar(n,&Ptype);
			break;
	    case 0x81:	buf->rwbytepar(n,&Pfreq);
			break;
	    case 0x82:	buf->rwbytepar(n,&Pq);
			break;
	    case 0x83:	buf->rwbytepar(n,&Pstages);
			break;
	    case 0x84:	buf->rwbytepar(n,&Pfreqtrack);
			break;
	};
    };

    
    if (buf->getmode()!=0) {
	unsigned char tmp=0xff;
	buf->rwbyte(&tmp);
    };
};
